ember-form-changeset-validations
==============================================================================

This addon helps creating form components logic based on [ember-changeset-validations](https://github.com/poteto/ember-changeset-validations).


Compatibility
------------------------------------------------------------------------------

* Ember.js v3.20 or above
* Ember CLI v3.20 or above
* Node.js v12 or above


Installation
------------------------------------------------------------------------------

```
ember install ember-form-changeset-validations
```


Features
------------------------------------------------------------------------------

### Form generation
Form generation with `ember generate form <formName>`.
- You can add --ask parameter to prompt fields to add to the template

### Base Form Component

```ts
import { BaseForm } from "ember-changeset-validations";

export interface MyChangesetDTO {
    name: string;
}

interface MyComponentArgs extends BaseFormArgs<MyChangesetDTO> {}

export default class MyFormComponent extends BaseForm<MyComponentArgs,MyChangesetDTO> {
}

```
#### Methods
- rollback
    - Rollback the changeset
- submit
    - Validates the changeset and triggers @saveFunction

### Typescript types
- Basic Typescript types for ember-changeset-validations
```json
"ember-changeset-validations": [
    "node_modules/ember-form-changeset-validations/types/ember-changeset-validations"
],
```
- Typed changeset interface
```ts
import { TypedBufferedChangeset } from "ember-changeset-validations";
// ...
const changeset = Changeset({}) as TypedBufferedChangeset<{
    name : string;
}>;

changeset.get("name"); // check prop name
changeset.set("name","blah"); // check prop name

// if unknown key, you can just specify the return type to keep type checking
changeset.get<string>("unknown.key");
changeset.set<string>("name","blah"); // check set prop type is string
changeset.set<string>("name", 0); // Error
```

### Utils
Utilitaries functions to work with POJO changesets.

#### toPojo 
Transforms an ember record or an ember record array to plain changeset.

```ts 
toPojo()<T extends Model>(object: T | ArrayProxy<T> | null,store: Store): Record<string, unknown> | Record<string, unknown>[]
```

##### Example

```ts
const article = await this.store.findRecord('article', id, {
      include: 'image',
});
const pojo = toPojo(article, this.store) as Record<string, unknown>;

// pojoize image relationship
pojo.image = pojo.image
    ? toPojo(this.store.peekRecord('image', pojo.image as string), this.store)
    : null;
```

## More examples

See [Dummy App](https://github.com/TRIPTYK/ember-form-changeset-validations/tree/main/tests/dummy)

License
------------------------------------------------------------------------------

This project is licensed under the [MIT License](LICENSE.md).
